local playerGuid = nil

local spellIds = {
	Transmute = 17187;
	SaltShaker = 19566;
	Mooncloth = 18560;
}

if (CDTData == nil) then
	CDTData = {}
end

local function setCharSkills()
	for i=1, GetNumSkillLines() do
		local skillName = GetSkillLineInfo(i);
		if (skillName == "Alchemy") then
			CDTData[playerGuid].cooldowns["Transmute"] = {}
		end
		if (skillName == "Leatherworking") then
			CDTData[playerGuid].cooldowns["SaltShaker"] = {}
		end
		if (skillName == "Tailoring") then
			CDTData[playerGuid].cooldowns["Mooncloth"] = {}
		end
	end
end

local function updateTimers()
	for key in pairs(CDTData[playerGuid].cooldowns) do
		local spellId = spellIds[key]
		local _start, _duration, enabled = GetSpellCooldown(spellId);
		CDTData[playerGuid].cooldowns[key] = {
			startTime = _start,
			duration = _duration,
		};
	end
end

function setCharDefault()
	local playerName = GetUnitName("player")
	local playerRealm = GetRealmName()
	local DefaultSettings = {
		enabled = false;
		name = playerName;
		realm = playerRealm;
		cooldowns = {};
	};
	if (playerGuid == nil) then
		playerGuid = UnitGUID("player")
	end
	CDTData[playerGuid] = DefaultSettings
	setCharSkills()
	updateTimers()
end

local function getCooldown(start, cooldown)
    --seconds
    local current = GetTime();
    local day = 0;
    local hour = 0;
    local minute = 0;
    local second = 0;
    local rday, rhour, rminute, rsecond;
    local SCD_PER_DAY = 86400;
    local SCD_PER_HOUR = 3600;
    local SCD_PER_MINUTE = 60;
    if start > 0 and cooldown > 0 then
        if (current - start) < cooldown then
            remaining = (start+cooldown) - current;
            rday = math.floor(remaining / SCD_PER_DAY);
            if rday > 0 then
                day = rday;
                remaining = remaining - (SCD_PER_DAY*day);
            end
            rhour = math.floor(remaining/SCD_PER_HOUR);
            if rhour > 0 then
                hour = rhour;
                remaining = remaining - (SCD_PER_HOUR*hour);
            end
            rminute = math.floor(remaining/SCD_PER_MINUTE);
            if rminute > 0 then
                minute = rminute;
                remaining = remaining - (SCD_PER_MINUTE*minute);
            end
			if day >= 1 or hour >= 6 then
                return "|cffff0000"..day.."d "..hour.."h "..minute.."min";
            else
                return "|cffffff00" .. day.."d "..hour.."h "..minute.."min";
            end
        end
    else
        return false
    end
end

local function getPrintData(playerData)
	local msg = "Profession Cooldown Timer Data:\n";
    local size = 0;
    if CDTData and CDTData ~= {} then
        for playerId, playerData in pairs(CDTData) do
			local playerName = playerData.name
			if (playerData["cooldowns"] and playerData["enabled"] == true) then
				for prof, profData in pairs(playerData["cooldowns"]) do
					cooldown = getCooldown(profData["startTime"], profData["duration"]);
					if not cooldown then
						cooldown = "|cff00ff00 Available or no data|r"
					end
					msg = msg .. "|cffffffff Char: ".. playerName .. " | " .. prof .. " Timer: |r".. cooldown .. "|n";
					size = size + 1;
				end
			end
        end
    end
	if (size == 0) then
		msg = "Cooldown Timer: No Data";
	end
	return msg, size
end

local function printData()
    if CDTData then
		local msg, size = getPrintData(playerData)
		print(msg)
    else
        print("Cooldown Timer has no data to print.")
    end
end

local PCDTFrame = CreateFrame("Frame");
PCDTFrame:RegisterEvent("PLAYER_ENTERING_WORLD");
PCDTFrame:RegisterEvent("PLAYER_LOGIN");
PCDTFrame:RegisterEvent("PLAYER_LEAVING_WORLD");

function PCDTFrame:OnEvent(event, arg1)
	if (event == "PLAYER_ENTERING_WORLD") then
		playerGuid = UnitGUID("player")
		C_Timer.After(3, printData)
		if (CDTData[playerGuid] and CDTData[playerGuid].enabled == true) then
			C_Timer.After(1, updateTimers)
		else
			print("Cooldown Tracker is not tracking cooldowns for this character. Type \"/cdt enable\" to enable tracking")
		end
	elseif (event == "PLAYER_LEAVING_WORLD") then
		if (CDTData[playerGuid] and CDTData[playerGuid].enabled == true) then
			updateTimers()
		end
	end
end

function showPCDTData()
    local msg = "";
    local size = 0;
    if CDTData then
        msg, size = getPrintData(playerData)
    else
        msg = "Cooldown Timer: No Data";
    end
    if size > 0 then
        CDT_initMainFrame(msg, size + 1);
    end
end

PCDTFrame:SetScript("OnEvent", PCDTFrame.OnEvent);

SLASH_CDT1 = "/cdt";

SlashCmdList["CDT"] = function(msg, editBox)
    if (msg == "enable") then
		setCharDefault()
		CDTData[playerGuid].enabled = true
        print("Cooldown Timer enabled for this character")
	elseif (msg == "disable") then
		CDTData[playerGuid].enabled = false
		print("Cooldown Timer disabled for this character")
	elseif (msg == "show" or msg == "s") then
		showPCDTData()
	elseif (msg == "print" or msg == "p") then
		printData()
	elseif (msg == "reset") then
		CDTData[playerGuid] = nil
		print("Cooldown Tracker data has been reset. Please reload UI for changes to take effect")
    else
        print("Uknown PCDT command " .. SLASH_CDT1 .." " .. msg )
		print("Type \"/cdt enable\" to enable CDT for this character.")
		print("Type \"/cdt show\" or \"/cdt s\" to show the CDT timer GUI.")
		print("Type \"/cdt print\" or \"/cdt p\" to print current PCDT timer data.")
		print("Type \"/cdt disable\" to disable cooldown tracking for this character.")
		print("Type \"/cdt reset\" to reset this character's timer data.")
    end
end
